# -*- coding: utf-8 -*-
import sys
from .inputs import String
from .inputs import Integer
from .inputs import Select
from .inputs import Boolean
from .inputs import MultiSelect
from .inputs import Branch
from .utils import numeric_options
from .utils import prepare_stylesheet


class AddAction(object):

    def __init__(self, form):
        self.form = form

    def __call__(self, key, obj):
        self.form._fields.append((key, obj))

    def string(self, key, label, **kwargs):
        obj = String(label, **kwargs)
        self.form._add(key, obj)
        return self.form

    def int(self, key, label, **kwargs):
        obj = Integer(label, **kwargs)
        self.form._add(key, obj)
        return self.form

    def select(self, key, label, choices, option_format=numeric_options, **kwargs):
        obj = Select(label, choices, option_format, **kwargs)
        self.form._add(key, obj)
        return self.form

    def multiselect(self, key, label, choices, done_label='Done', option_format=numeric_options, **kwargs):
        obj = MultiSelect(label, choices, done_label, option_format, **kwargs)
        self.form._add(key, obj)
        return self.form

    def bool(self, key, label, **kwargs):
        obj = Boolean(label, **kwargs)
        self.form._add(key, obj)
        return self.form

    def branch(self, handler, *args, **kwargs):
        obj = Branch(handler, *args, **kwargs)
        self.form._add(id(obj), obj)
        return self.form


class Form(object):

    @property
    def add(self):
        action = AddAction(self)
        return action

    def __init__(self):
        self._fields = []

    def _add(self, key, obj):
        self._fields.append((key, obj))

    def __len__(self):
        return len(self._fields)

    def __iter__(self):
        for k, v in iter(self._fields):
            if not isinstance(v, Branch):
                yield k, v.value

    def __getattr__(self, key):
        try:
            # we were using an ordereddict so access was pretty close to
            # O(1), switched to lists in order to facilitate branching
            # this lookup will be O(N)
            return next(x[1] for x in self._fields if x[0] == key)
        except StopIteration:
            raise AttributeError



import colorama


colorama.init()


class Style(object):

    reset_all = colorama.Style.RESET_ALL

    @classmethod
    def styles_for_key(cls, key, stylesheet):
        styles = {}
        context = stylesheet

        try:
            styles.update(context['selectors']['body']['value'])
        except KeyError:
            pass

        for part in key.split('.'):
            try:
                context = context['selectors'][part]
                styles.update(context['value'])

            except KeyError:
                break

        return cls(styles)

    def __init__(self, data):
        self.data = data

    @property
    def color(self):
        try:
            return getattr(colorama.Fore, self.data['color'].upper())
        except (KeyError, AttributeError):
            return ''

    @property
    def background_color(self):
        try:
            return getattr(colorama.Back, self.data['background_color'].upper())
        except (KeyError, AttributeError):
            return ''

    @property
    def font_weight(self):

        try:
            font_weight = self.data['font_weight']
        except KeyError:
            return ''

        choices = {'normal': colorama.Style.NORMAL,
                   'bold': colorama.Style.BRIGHT,
                   'lighter': colorama.Style.DIM}

        return choices.get(font_weight, colorama.Style.NORMAL)

    def __call__(self, value):
        # Fore: BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE, RESET.
        # Back: BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE, RESET.
        # Style: DIM, NORMAL, BRIGHT, RESET_ALL
        return self.reset_all + self.color + \
               self.background_color + \
               self.font_weight + \
               value + \
               self.reset_all


class CSSTokens(object):

    SELECTOR = 'SELECTOR'
    BODY_START = 'BODY_START'
    BODY_END = 'BODY_END'
    PROPERTY = 'PROPERTY'
    PROPERTY_VALUE = 'PROPERTY_VALUE'


class CSSParser(object):

    @staticmethod
    def parse_string(value):
        parser = CSSParser()
        return parser.parse(value)

    def parse(self, value):
        sheet = {'selectors':{}}

        context = sheet
        property_name = None

        for token, value in self.tokenize(value):
            if token == CSSTokens.SELECTOR:
                data = self.format(value)

                if data:
                    context = context.setdefault('selectors', {})
                    context = context.setdefault(data, {'value': {}})

            elif token == CSSTokens.BODY_START:
                data = self.format(value)
                if data:
                    context = context.setdefault('selectors', {})
                    context = context.setdefault(data, {'value': {}})

            elif token == CSSTokens.PROPERTY:
                property_name = self.format(value)

            elif token == CSSTokens.PROPERTY_VALUE:
                context['value'][property_name] = self.format(value)
                property_name = None

            elif token == CSSTokens.BODY_END:
                data = self.format(value)
                if data:
                    context['value'][property_name] = data

                context = sheet

        return sheet

    def format(self, value):
        return value \
               .strip() \
               .replace('-', '_') \
               .lower()

    def tokenize(self, value):
        buf = ''

        for char in value:

            if char == '.':
                yield CSSTokens.SELECTOR, buf
                buf = ''

            elif char == '{':
                yield CSSTokens.BODY_START, buf
                buf = ''

            elif char == '}':
                yield CSSTokens.BODY_END, buf
                buf = ''

            elif char == ':':
                yield CSSTokens.PROPERTY, buf
                buf = ''

            elif char == ';':
                yield CSSTokens.PROPERTY_VALUE, buf
                buf = ''
            else:
                buf = buf + str(char)






import sys
from .form import Form
from .inputs import String
from .inputs import Integer
from .inputs import Select
from .inputs import MultiSelect
from .inputs import Boolean
from .inputs import Branch
from .inputs import Notification
from .utils import prepare_stylesheet
from .runners import console



class Prompt(object):

    def process_value(self, value):
        return value

    def apply_default(self, value):
        if len(value) == 0 and self.input.default is not None:
            return self.input.default

        return value



import sys

_ver = sys.version_info
is_py2 = (_ver[0] == 2)
is_py3 = (_ver[0] == 3)

if is_py3:
    unichr = chr
    unicode = str
else:
    unichr = unichr
    unicode = unicode


def iteritems(value):
    if is_py2:
        return value.iteritems()

    return value.items()


def itervalues(value):
    if is_py2:
        return value.itervalues()

    return value.values()


def input(prompt):
    if is_py2:
        return raw_input(prompt.encode(sys.stdout.encoding))
    else:
        return input(prompt)



# -*- coding: utf-8 -*-
from .utils import numeric_options


class Branch(object):
    def __init__(self, handler, *args, **kwargs):
        self.handler = handler
        self.args = args
        self.kwargs = kwargs

    def __call__(self, form):
        branch = self.handler(form, *self.args, **self.kwargs)

        if not branch:
            return

        index = form._fields.index((id(self), self)) + 1

        for each in iter(branch._fields):
            form._fields.insert(index, each)
            index = index + 1


class BaseInput(object):

    def __init__(self, label, default=None, notifications=()):
        self.label = label
        self.default = default
        self.notifications = map(Notification, notifications)
        self.value = None


class Notification(BaseInput):
    pass


class String(BaseInput):
    pass


class Integer(BaseInput):
    pass


class Boolean(BaseInput):
    def __init__(self, label, default=True, **kwargs):
        super(Boolean, self).__init__(label, default=default, **kwargs)


class Select(BaseInput):

    def __init__(self, label, choices, option_format=numeric_options, **kwargs):
        super(Select, self).__init__(label, **kwargs)
        self.choices = option_format(choices)


class MultiSelect(Select):

    def __init__(self, label, choices, done_label='Done', option_format=numeric_options, **kwargs):
        options = list(choices)
        options += (done_label, )
        super(MultiSelect, self).__init__(label, options, option_format, **kwargs)





import pkg_resources
from .styles import CSSParser


DEFAULT_STYLESHEET_CACHE = None


def numeric_options(options):

    max = len(options) + 1
    numbers = range(1, max)
    # call to list if for python 3
    return list(zip(numbers, options))


def prepare_stylesheet(value=None):
    global DEFAULT_STYLESHEET_CACHE

    if value:
        return CSSParser.parse_string(value)

    if DEFAULT_STYLESHEET_CACHE:
        return DEFAULT_STYLESHEET_CACHE

    stream = pkg_resources.resource_stream(
        'promptly.resources',
        'default.css'
    )

    DEFAULT_STYLESHEET_CACHE = CSSParser.parse_string(stream.read())

    return DEFAULT_STYLESHEET_CACHE



import sys
import signal

# on windows pyreadline will be installed
# pyreadline installs a file under the name
# readline.py in site-packages
import readline

from promptly.styles import Style
from promptly.utils import prepare_stylesheet
from promptly.renderers import console
from promptly.compat import input as get_input
from promptly import Notification


def run(form, prefix=None, stylesheet=None):
    runner = ConsoleRunner(form, prefix=prefix, stylesheet=stylesheet)
    runner.run()


def notification(text, prefix=None, stylesheet=None):
    styles = prepare_stylesheet(stylesheet)
    prefix = '' if prefix is None else prefix
    message = Notification(text)

    obj = console.NotificationPrompt(
        runner=None,
        input=message,
        prefix=prefix,
        stylesheet=styles)

    sys.stdout.write('\n%s' % obj.prompt)
    sys.stdout.flush()


class ConsoleRunner(object):

    def __init__(self, form, prefix=None, stylesheet=None):
        self.form = form
        self.prefix = '' if prefix is None else prefix
        self.stylesheet = prepare_stylesheet(stylesheet)
        signal.signal(signal.SIGINT, self._on_sigint)

    # signal handlers
    def _on_sigint(self, signal, frame):
        sys.stdout.write(Style.reset_all)
        print ('\nYou Quit! You\'re a quitter! Boo!\n')
        sys.exit(0)

    def branch(self, label, input):
        input(self.form)

    def notification(self, label, input, prefix=True):
        obj = self.factory(console.NotificationPrompt, input)

        if not prefix:
            obj.prefix = ''

        return obj

    def string(self, label, input):
        return self.factory(console.StringPrompt, input)

    def boolean(self, label, input):
        return self.factory(console.BooleanPrompt, input)

    def select(self, label, input):
        return self.factory(console.SelectPrompt, input)

    def multiselect(self, label, input):
        return self.factory(console.MultiSelectPrompt, input)

    def integer(self, label, input):
        return self.factory(console.IntegerPrompt, input)

    def factory(self, cls, input):
        return cls(self, input, self.prefix, self.stylesheet)

    def loop(self):
        while 1:
            result = None
            prompt = (yield)

            while 1:
                result = self.render(
                    prompt.prompt,
                    default=prompt.default)

                result = prompt.apply_default(result)

                try:
                    result = prompt.process_value(result)
                except:
                    continue
                break
            yield result

    def run(self):
        loop = self.loop()
        next(loop)

        for label, input in iter(self.form._fields):
            key = input.__class__.__name__.lower()
            prompt = getattr(self, key)(label, input)
            if prompt:
                data = loop.send(prompt)
                input.value = data
                next(loop)

    def prompt_format(self, prompt):
        stylesheet = self.stylesheet
        styles_action = Style.styles_for_key('action', stylesheet)
        styles_input = Style.styles_for_key('input', stylesheet)

        return '\n%(prompt)s\n%(action)s%(input)s' % {
            'prompt': prompt,
            'action': styles_action('> '),
            'input': styles_input.color + \
                     styles_input.background_color + \
                     styles_input.font_weight
        }

    def render(self, value, default=None):
        # http://stackoverflow.com/questions/2533120/show-default-value-for-editing-on-python-input-possible/2533142#2533142
        # http://stackoverflow.com/questions/3327524/crossplatform-method-for-inserting-text-into-raw-input-to-avoid-readine-in-pyt
        # windows: https://pypi.python.org/pypi/readline
        #          https://pypi.python.org/pypi/pyreadline/2.0
        #
        # Under windows we don't get the set_startup_hook functionality
        # So the defaults will not be displayed.
        # The prompts will be rendered differently for windows based clients

        result = None
        if default:
            readline.set_startup_hook(lambda: readline.insert_text(default))
            try:
                result = get_input(self.prompt_format(value))
            finally:
                readline.set_startup_hook()
        else:
            result = get_input(self.prompt_format(value))

        return result



#!/usr/bin/env python
from setuptools import setup


def get_requires():
    reqs = ['colorama']
    try:
        import readline
    except ImportError:
        reqs.append('pyreadline')

    return reqs

with open('README.md') as f:
    readme = f.read()

with open('LICENSE') as f:
    license = f.read()

packages = [
    'promptly',
    'promptly.renderers',
    'promptly.runners'
]


requires = get_requires()

setup(
    name='promptly',
    version='0.5.2',
    description='Console Prompting',
    long_description=readme,
    author='Aubrey Taylor <aubricus@gmail.com>, Adam Venturella <aventurella@gmail.com>',
    author_email='aubricus@gmail.com, aventurella@gmail.com',
    url='https://github.com/aventurella/promptly',
    license=license,
    packages=packages,
    package_data={'': ['LICENSE'],
                  'promptly': ['resources/*.*']},
    include_package_data=True,
    install_requires=requires,
    package_dir={'promptly': 'promptly'},
    classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.1',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
    ),
)



from __future__ import unicode_literals
import sys
from promptly.styles import Style
from promptly.compat import unichr
from promptly.compat import unicode
from .core import Prompt


class ConsolePrompt(Prompt):
    def __init__(self, runner, input, prefix, stylesheet):
        self.runner = runner
        self.input = input
        self.prefix = prefix
        self.stylesheet = stylesheet

    @property
    def default(self):
        if self.input.default:
            return str(self.input.default)
        return None

    def append_notifications(self, prompt, notifications):
        styles_footer = Style.styles_for_key('notification.footer', self.stylesheet)

        notices = []
        dot = unichr(0x00b7)

        for each in notifications:
            x = self.runner.notification('', each, prefix=False)
            notices.append(x.prompt.rjust(len(self.prefix), ' '))

        if notices:
            prompt = '%s\n%s' % (prompt, '\n'.join(notices))
            prompt = '%s\n%s' % (prompt, styles_footer(dot.ljust(3, dot)))

        return prompt


class NotificationPrompt(ConsolePrompt):

    @property
    def prompt(self):
        input = self.input
        stylesheet = self.stylesheet
        prefix = self.prefix

        styles_prefix = Style.styles_for_key('prefix', stylesheet)
        styles_label = Style.styles_for_key('notification.label', stylesheet)

        return '%s%s' % (styles_prefix(prefix),
                         styles_label(input.label))


class StringPrompt(ConsolePrompt):
    @property
    def prompt(self):
        input = self.input
        stylesheet = self.stylesheet
        prefix = self.prefix

        styles_prefix = Style.styles_for_key('prefix', stylesheet)
        styles_label = Style.styles_for_key('string.label', stylesheet)

        styles_default_wrapper = Style.styles_for_key(
            'string.default_wrapper', stylesheet)

        styles_default_value = Style.styles_for_key(
            'string.default_value', stylesheet)

        if not input.default:
            prompt = '%s%s' % (styles_prefix(prefix),
                               styles_label(input.label))
        else:
            try:
                # pyreadline does not support set_startup_hook
                # so we do the next best thing when it comes to defaults
                import pyreadline
                prompt = '%s%s %s%s%s' % (
                    styles_prefix(prefix),
                    styles_label(input.label),
                    styles_default_wrapper('['),
                    styles_default_value(input.default),
                    styles_default_wrapper(']'))
            except ImportError:
                prompt = '%s%s' % (styles_prefix(prefix),
                                   styles_label(input.label))

        return self.append_notifications(prompt, input.notifications)


class IntegerPrompt(ConsolePrompt):
    def process_value(self, value):
        try:
            return int(value)
        except ValueError:
            raise

    @property
    def prompt(self):
        input = self.input
        stylesheet = self.stylesheet
        prefix = self.prefix

        styles_prefix = Style.styles_for_key('prefix', stylesheet)
        styles_label = Style.styles_for_key('integer.label', stylesheet)

        styles_default_wrapper = Style.styles_for_key(
            'integer.default_wrapper', stylesheet)

        styles_default_value = Style.styles_for_key(
            'integer.default_value', stylesheet)

        if not input.default:
            prompt = '%s%s' % (styles_prefix(prefix),
                               styles_label(input.label))

        else:
            try:
                # pyreadline does not support set_startup_hook
                # so we do the next best thing when it comes to defaults
                import pyreadline
                prompt = '%s%s %s%s%s' % (
                    styles_prefix(prefix),
                    styles_label(input.label),
                    styles_default_wrapper('['),
                    styles_default_value(input.default),
                    styles_default_wrapper(']'))
            except ImportError:
                prompt = '%s%s' % (styles_prefix(prefix),
                                   styles_label(input.label))

        return self.append_notifications(prompt, input.notifications)


class BooleanPrompt(ConsolePrompt):

    def process_value(self, value):
        boolean_states = {
        True: True, '1': True, 'yes': True, 'true': True, 'on': True, 'y': True, 't': True,
        False: False, '0': False, 'no': False, 'false': False, 'off': False, 'n': False, 'f': False
        }

        candidate = value.lower() if type(value) == str else value

        try:
            return boolean_states[candidate]
        except KeyError:
            raise

    @property
    def default(self):
        return None

    @property
    def prompt(self):
        input = self.input
        stylesheet = self.stylesheet
        prefix = self.prefix

        styles_prefix = Style.styles_for_key('prefix', stylesheet)
        styles_label = Style.styles_for_key('boolean.label', stylesheet)

        styles_default_wrapper = Style.styles_for_key(
            'boolean.default_wrapper', stylesheet)

        styles_default_value = Style.styles_for_key(
            'boolean.default_value', stylesheet)

        styles_other_value = Style.styles_for_key(
            'boolean.other_value', stylesheet)

        styles_seperator = Style.styles_for_key(
            'boolean.seperator', stylesheet)

        prompt = styles_label(input.label)

        if input.default:
            prompt = '%s%s %s%s%s%s%s' % (
                styles_prefix(prefix),
                prompt,
                styles_default_wrapper('['),
                styles_default_value('Y'),
                styles_seperator('/'),
                styles_other_value('n'),
                styles_default_wrapper(']'))
        else:
            prompt = '%s%s %s%s%s%s%s' % (
                styles_prefix(prefix),
                prompt,
                styles_default_wrapper('['),
                styles_other_value('y'),
                styles_seperator('/'),
                styles_default_value('N'),
                styles_default_wrapper(']'))

        return prompt


class SelectPrompt(ConsolePrompt):
    @property
    def default(self):
        return None

    def process_value(self, value):
        result = [x for x in self.input.choices if str(x[0]) == str(value)]

        if not result:
            raise ValueError

        return result[0]

    @property
    def prompt(self):
        input = self.input
        stylesheet = self.stylesheet
        prefix = self.prefix

        styles_prefix = Style.styles_for_key('prefix', stylesheet)
        styles_label = Style.styles_for_key('choices.label', stylesheet)
        styles_default_wrapper = Style.styles_for_key(
            'choices.default_wrapper', stylesheet)

        styles_default_value = Style.styles_for_key(
            'choices.default_value', stylesheet)

        styles_option_key = Style.styles_for_key(
            'choices.option_key', stylesheet)

        styles_option_value = Style.styles_for_key(
            'choices.option_value', stylesheet)

        styles_seperator = Style.styles_for_key(
            'choices.seperator', stylesheet)

        styles_action = Style.styles_for_key('choices.action', stylesheet)

        prompt = '%s' % styles_label(input.label)

        choices = []

        border_len = 0
        for key, value in input.choices:
            border_len = max(len(value), border_len)
            choices.append('%s%s %s' % (
                styles_option_key(unicode(key)),
                styles_seperator(')'),
                styles_option_value(value)))

        prompt = '%s%s' % (
            styles_prefix(prefix),
            prompt)

        if input.default:
            prompt = '%s %s%s%s' % (prompt,
                styles_default_wrapper('['),
                styles_default_value(unicode(input.default)),
                styles_default_wrapper(']'))

        prompt = self.append_notifications(prompt, input.notifications)

        dot = unichr(0x00b7)
        styles_notification_footer = Style.styles_for_key(
            'notification.footer', stylesheet)

        prompt = '%s\n%s%s' % (
            prompt,
            '\n'.join(choices),
            styles_notification_footer('\n'.ljust(4, dot)))

        return prompt


class MultiSelectPrompt(ConsolePrompt):
    def __init__(self, runner, input, prefix, stylesheet):
        super(MultiSelectPrompt, self).__init__(runner, input, prefix, stylesheet)
        self.values = set()

    def process_value(self, value):
        choices = self.input.choices
        choices_count = len(choices)

        try:
            obj = next(x for x in choices if str(x[0]) == str(value))
        except StopIteration:
            pass

        # Did we get the "Done" option?
        if choices.index(obj) == (choices_count - 1):
            return tuple(self.values)

        if obj in self.values:
            self.values.remove(obj)
        else:
            self.values.add(obj)

        # force the run loop to contine
        raise Exception

    @property
    def default(self):
        return None

    @property
    def prompt(self):
        input = self.input
        stylesheet = self.stylesheet
        prefix = self.prefix

        styles_prefix = Style.styles_for_key('prefix', stylesheet)
        styles_label = Style.styles_for_key('choices.label', stylesheet)

        styles_default_wrapper = Style.styles_for_key(
            'choices.default_wrapper', stylesheet)

        styles_default_value = Style.styles_for_key(
            'choices.default_value', stylesheet)

        styles_option_key = Style.styles_for_key(
            'choices.option_key', stylesheet)

        styles_option_value = Style.styles_for_key(
            'choices.option_value', stylesheet)

        styles_seperator = Style.styles_for_key(
            'choices.seperator', stylesheet)

        styles_action = Style.styles_for_key(
            'choices.action', stylesheet)

        styles_selection = Style.styles_for_key(
            'choices.selection', stylesheet)

        prompt = '%s' % styles_label(input.label)

        choices = []
        choices_count = len(input.choices)

        prompt = '%s%s' % (
            styles_prefix(prefix),
            prompt)

        for i, each in enumerate(input.choices):
            key, value = each
            prefix = '[ ] '

            if i == (choices_count - 1):
                prefix = ''
            elif each in self.values:
                prefix = '[%s] ' % styles_selection('x')

            choices.append('%s%s %s' % (
                styles_option_key(unicode(key)),
                styles_seperator(')'),
                styles_option_value('%s%s' % (prefix, value))))

        prompt = self.append_notifications(prompt, input.notifications)

        dot = unichr(0x00b7)
        styles_notification_footer = Style.styles_for_key(
            'notification.footer', stylesheet)

        prompt = '%s\n%s%s' % (
            prompt,
            '\n'.join(choices),
            styles_notification_footer('\n'.ljust(4, dot)))

        return prompt
